@@ -26,9 +26,9 @@
 #include <util/log.h>
 #include <util/glib.h>
 
-#include <jsapi.h>
-
-#include <gjs/gjs.h>
+#include <gjs/gjs-module.h>
+#include <gjs/importer.h>
+#include <gjs/compat.h>
 
 #include <string.h>
 
@@ -284,11 +284,16 @@ load_module_init(JSContext  *context,
         }
     }
 
-    module_obj = JS_ConstructObject(context, NULL, NULL, NULL);
+    module_obj = JS_NewObject(context, NULL, NULL, NULL);
     if (module_obj == NULL) {
         return JS_FALSE;
     }
 
+    /* https://bugzilla.mozilla.org/show_bug.cgi?id=599651 means we
+     * can't just pass in the global as the parent */
+    JS_SetParent(context, module_obj,
+                 gjs_get_import_global (context));
+
     /* Define module in importer for future use and to avoid module_obj
      * object to be garbage collected during the evaluation of the script */
     JS_DefineProperty(context, in_object,
@@ -360,15 +365,10 @@ load_module_elements(JSContext *context,
             return;
         }
 
-        while (idp != JSVAL_VOID) {
-            jsval nameval;
+        while (!JSID_IS_VOID(idp)) {
             const char *name;
 
-            if (!JS_IdToValue(context, idp, &nameval)) {
-                continue;
-            }
-
-            if (!gjs_get_string_id(nameval, &name)) {
+            if (!gjs_get_string_id(context, idp, &name)) {
                 continue;
             }
 
@@ -728,6 +728,9 @@ importer_new_enumerate(JSContext  *context,
     ImporterIterator *iter;
 
     switch (enum_op) {
+#ifdef HAVE_MOZJS_2
+    case JSENUMERATE_INIT_ALL:
+#endif
     case JSENUMERATE_INIT: {
         Importer *priv;
         JSObject *search_path;
@@ -739,7 +742,7 @@ importer_new_enumerate(JSContext  *context,
             *state_p = JSVAL_NULL;
 
         if (id_p)
-            *id_p = JSVAL_ZERO;
+            *id_p = INT_TO_JSID(0);
 
         priv = priv_from_js(context, object);
         if (!priv)
@@ -846,7 +849,7 @@ importer_new_enumerate(JSContext  *context,
             *state_p = PRIVATE_TO_JSVAL(iter);
 
         if (id_p)
-            *id_p = INT_TO_JSVAL(iter->elements->len);
+            *id_p = INT_TO_JSID(iter->elements->len);
 
         break;
     }
@@ -859,6 +862,9 @@ importer_new_enumerate(JSContext  *context,
             return JS_FALSE;
         }
 
+        if (*state_p == JSVAL_NULL) /* Iterating prototype */
+            return JS_TRUE;
+
         iter = JSVAL_TO_PRIVATE(*state_p);
 
         if (iter->index < iter->elements->len) {
@@ -907,17 +913,17 @@ importer_new_enumerate(JSContext  *context,
 static JSBool
 importer_new_resolve(JSContext *context,
                      JSObject  *obj,
-                     jsval      id,
+                     jsid       id,
                      uintN      flags,
                      JSObject **objp)
 {
     Importer *priv;
     const char *name;
-    JSContext *load_context;
 
     *objp = NULL;
 
-    name = gjs_string_get_ascii(id);
+    if (!gjs_get_string_id(context, id, &name))
+        return JS_FALSE;
 
     /* let Object.prototype resolve these */
     if (strcmp(name, "valueOf") == 0 ||
@@ -931,21 +937,13 @@ importer_new_resolve(JSContext *context,
     if (priv == NULL)
         return JS_TRUE; /* we are the prototype, or have the wrong class */
 
-    /* We always import in the special load context. */
-    load_context = gjs_runtime_get_load_context(JS_GetRuntime(context));
-    JS_BeginRequest(load_context);
-    if (do_import(load_context, obj, priv, name)) {
+    JS_BeginRequest(context);
+    if (do_import(context, obj, priv, name)) {
         *objp = obj;
-        JS_EndRequest(load_context);
+        JS_EndRequest(context);
         return JS_TRUE;
     } else {
-        /* Move the exception to the calling context from load context.
-         */
-        if (!gjs_move_exception(load_context, context)) {
-            /* set an exception since none was set */
-            gjs_throw(context, "No exception was set, but import failed somehow");
-        }
-        JS_EndRequest(load_context);
+        JS_EndRequest(context);
         return JS_FALSE;
     }
 }
@@ -1037,7 +1035,7 @@ importer_new(JSContext    *context)
     Importer *priv;
     JSObject *global;
 
-    global = JS_GetGlobalObject(context);
+    global = gjs_get_import_global(context);
 
     if (!gjs_object_has_property(context, global, gjs_importer_class.name)) {
         JSObject *prototype;
@@ -1072,7 +1070,7 @@ importer_new(JSContext    *context)
                   gjs_importer_class.name, prototype);
     }
 
-    importer = JS_ConstructObject(context, &gjs_importer_class, NULL, NULL);
+    importer = JS_ConstructObject(context, &gjs_importer_class, NULL, global);
     if (importer == NULL)
         gjs_fatal("No memory to create ns object");
 
@@ -1185,20 +1183,20 @@ gjs_define_importer(JSContext    *context,
  * we just ignore all calls after the first and hope the args are the same.
  */
 JSBool
-gjs_create_root_importer(JSRuntime   *runtime,
+gjs_create_root_importer(JSContext   *context,
                          const char **initial_search_path,
                          gboolean     add_standard_search_path)
 {
-    JSContext *context;
+    JSObject *global;
 
-    context = gjs_runtime_get_load_context(runtime);
+    global = gjs_get_import_global(context);
 
     JS_BeginRequest(context);
 
     if (!gjs_object_has_property(context,
-                                 JS_GetGlobalObject(context),
+                                 global,
                                  "imports")) {
-        if (gjs_define_importer(context, JS_GetGlobalObject(context),
+        if (gjs_define_importer(context, global,
                                 "imports",
                                 initial_search_path, add_standard_search_path) == NULL) {
             JS_EndRequest(context);
@@ -1220,16 +1218,16 @@ gjs_define_root_importer(JSContext   *context,
                          JSObject    *in_object,
                          const char  *importer_name)
 {
-    JSContext *load_context;
+    JSObject *global;
     jsval value;
     JSBool success;
 
     success = JS_FALSE;
-    load_context = gjs_runtime_get_load_context(JS_GetRuntime(context));
-    JS_BeginRequest(load_context);
+    global = gjs_get_import_global(context);
+    JS_BeginRequest(context);
 
-    if (!gjs_object_require_property(load_context,
-                                     JS_GetGlobalObject(load_context), "global object",
+    if (!gjs_object_require_property(context,
+                                     global, "global object",
                                      "imports", &value) ||
         !JSVAL_IS_OBJECT(value)) {
         gjs_debug(GJS_DEBUG_IMPORTER, "Root importer did not exist, couldn't get from load context; must create it");
@@ -1247,6 +1245,6 @@ gjs_define_root_importer(JSContext   *context,
 
     success = JS_TRUE;
  fail:
-    JS_EndRequest(load_context);
+    JS_EndRequest(context);
     return success;
 }
