@@ -24,8 +24,8 @@
 #ifndef __GJS_NATIVE_H__
 #define __GJS_NATIVE_H__
 
-#if !defined (__GJS_GJS_H__) && !defined (GJS_COMPILATION)
-#error "Only <gjs/gjs.h> can be included directly."
+#if !defined (__GJS_GJS_MODULE_H__) && !defined (GJS_COMPILATION)
+#error "Only <gjs/gjs-module.h> can be included directly."
 #endif
 
 #include <glib.h>
